/* This is the widget wich draws the outcome of the precision test.
 * For this, it tests whether three points lie on a line or whether
 * they make a left or right turn.
 * 
 * author: Lukas Novosad - novosluk@fel.cvut.cz
 * date: 28/9/2010
 *
 */

#include "orientationwidget.h"

OrientationWidget::OrientationWidget(QWidget *parent)
    : QWidget(parent)
{
    ui.setupUi(this);

    QObject::connect(this, SIGNAL(repaintImage()), this, SLOT(repaint()));

    // fix the size of the widget to our needs
    setMinimumSize(256,256);
    setMaximumSize(256,256);

    // set an initial value for the epsilon exponent
    _epsilonExponent = -53  ;
      
    // set an initial precision
    _precision = CD;

    mujObrazek = new QImage(256, 256, QImage::Format_RGB888);
    updateImage();  // first draw...
}

OrientationWidget::~OrientationWidget()
{

}

void OrientationWidget::paintEvent(QPaintEvent * paintEvent)
{
    QPainter painter(this);
    painter.drawImage((int)0, (int)0, *mujObrazek); //paintImage from memory
}

/*
    This method implements the painting into the image in tne memory.
    It overrides the abstract definition from the QWidget class.
*/
void OrientationWidget::updateImage()
{
    QPainter painter(mujObrazek);

    painter.eraseRect(240,240,10,10);
        
    switch(_precision)
    {
        case CF: //Cartesian<float> 
            {
                Point_CF q(12, 12);
                Point_CF r(24, 24);
                float u = pow(2.0,_epsilonExponent);
                for(int j=0; j<256; j++)
                    for(int i=0; i<256; i++)
                    {
                        Point_CF p(0.5+i*u, 0.5+j*u);
                        drawPixel(CGAL::orientation(p,q,r), i, j, &painter);
                    }
                break;
            }
        case CD:  //Cartesian<double>
            {
                Point_CD q(12, 12);  
                Point_CD r(24, 24);
                double u = pow(2.0,_epsilonExponent);
                for(int j=0; j<256; j++)
                    for(int i=0; i<256; i++)
                    {
                        Point_CD p(0.5+i*u, 0.5+j*u);
                        drawPixel(CGAL::orientation(p,q,r), i, j, &painter);
                    }
                break;
            }
        // ======== BEGIN OF SOLUTION - TASK 1-3 ======== //
        // TODO: add the additional cases for the new point types
        // ========  END OF SOLUTION - TASK 1-3  ======== //
    }

    //}

    // draw the diagonal line in black
    painter.setPen(Qt::black);
    painter.drawLine(0,255,255,0);


    emit repaintImage(); 
}

void OrientationWidget::drawPixel(CGAL::Sign orientation, int i, int j, QPainter * painter)
{
    switch(orientation)
    {
        case CGAL::LEFT_TURN:
            painter->setPen(Qt::red);
            painter->drawPoint(i,255-j);
            break;
        case CGAL::RIGHT_TURN:
            painter->setPen(Qt::blue);
            painter->drawPoint(i,255-j);
            break;
        case CGAL::COLLINEAR:
            painter->setPen(Qt::yellow);
            painter->drawPoint(i,255-j);
            break;
    }
}
void OrientationWidget::setEpsilonExponent(int value)
{
    if(_epsilonExponent != value) 
    {
      _epsilonExponent = value;
      updateImage();
    }
}

int OrientationWidget::getEpsilonExponent()
{
    return _epsilonExponent;
}

void OrientationWidget::setPrecision(Precision value)
{
    if(_precision != value) 
    {
      _precision = value;
      updateImage();  // update the image in the memory 
    }
}

