/* This is the widget wich draws the outcome of the precision test.
 * For this, it tests whether three points lie on a line or if
 * they make a left or right turn.
 * 
 * author: Lukas Novosad - novosluk@fel.cvut.cz
 * date: 28/9/2010
 * update: Petr Felkel - felkepet@fel.cvut.cz, 1.10.2012
 * - paint to image instead directly to the screen - speedup by repaint event save
 */

#ifndef ORIENTATIONWIDGET_H
#define ORIENTATIONWIDGET_H

// Qt includes
#include <QWidget>
#include <QPainter>
#include <QPaintEvent>
#include "ui_orientationwidget.h"

// CGAL includes
#include <CGAL/Homogeneous.h>
#include <CGAL/Cartesian.h>
#include <CGAL/Point_2.h>
#include <CGAL/predicates_on_points_2.h>

// include necessary headers for data types used 
// ======== BEGIN OF SOLUTION - TASK 1-5 ======== //
// ========  END OF SOLUTION - TASK 1-5  ======== //

// some typdefs to make CGAL code more readable
// REPRESENTATION data types
typedef CGAL::Cartesian<float>			Rep_cart_float;
typedef CGAL::Cartesian<double>			Rep_cart_double;

// DEFINE your REPRESENTATION data types
// ======== BEGIN OF SOLUTION - TASK 1-6 ======== //
// ========  END OF SOLUTION - TASK 1-6  ======== //

// POINTS data type
typedef CGAL::Point_2<Rep_cart_float>   Point_CF;
typedef CGAL::Point_2<Rep_cart_double>  Point_CD;

// DEFINE your POINTS data type 
// ======== BEGIN OF SOLUTION - TASK 1-7 ======== //
// ========  END OF SOLUTION - TASK 1-7  ======== //

// the different kinds of precision so far:
// CF: cartesian float
// CD: cartesian double
// ======== BEGIN OF SOLUTION - TASK 1-8 ======== //
enum Precision {CF,CD, CG, CGP, CGQ, HI, HF, HD, HLD, HG, HGP, HGQ}; // ADD enum for your new types
// ========  END OF SOLUTION - TASK 1-8  ======== //

class OrientationWidget : public QWidget
{
    Q_OBJECT

public:
    OrientationWidget(QWidget *parent = 0);
    ~OrientationWidget();

    int getEpsilonExponent();

public slots:
    void setEpsilonExponent(int value);
    void setPrecision(Precision value);

protected:

    void paintEvent(QPaintEvent *);
    void drawPixel(CGAL::Sign orientation, int i, int j, QPainter * painter);
    void updateImage();

signals:
    void repaintImage();

private:
    Ui::OrientationWidgetClass ui;

    // the exponent for the epsilon value
    int _epsilonExponent;
    // the precision used for the test
    Precision _precision;
    QImage *mujObrazek;

};

#endif // ORIENTATIONWIDGET_H